@@ -25,6 +25,7 @@ from ai.backend.common.types import (
 from ai.backend.common.utils import current_loop, nmget
 
 from .. import __version__
+from ..alloc_map import AllocationStrategy
 from ..resources import (
     AbstractAllocMap,
     AbstractComputeDevice,
@@ -45,7 +46,7 @@ from ..vendor.linux import libnuma
 from .agent import Container
 from .resources import get_resource_spec_from_container
 
-log = BraceStyleAdapter(logging.getLogger(__name__))
+log = BraceStyleAdapter(logging.getLogger(__spec__.name))  # type: ignore[name-defined]
 
 
 async def fetch_api_stats(container: DockerContainer) -> Optional[Dict[str, Any]]:
@@ -175,9 +176,9 @@ class CPUPlugin(AbstractComputePlugin):
         container_ids: Sequence[str],
     ) -> Sequence[ContainerMeasurement]:
         async def sysfs_impl(container_id):
-            cpu_prefix = f"/sys/fs/cgroup/cpuacct/docker/{container_id}/"
+            cpu_path = ctx.agent.get_cgroup_path("cpuacct", container_id)
             try:
-                cpu_used = read_sysfs(cpu_prefix + "cpuacct.usage", int) / 1e6
+                cpu_used = read_sysfs(cpu_path / "cpuacct.usage", int) / 1e6
             except IOError as e:
                 log.warning(
                     "cannot read stats: sysfs unreadable for container {0}\n{1!r}",
@@ -459,11 +460,11 @@ class MemoryPlugin(AbstractComputePlugin):
             # return total_size
 
         async def sysfs_impl(container_id):
-            mem_prefix = f"/sys/fs/cgroup/memory/docker/{container_id}/"
-            io_prefix = f"/sys/fs/cgroup/blkio/docker/{container_id}/"
+            mem_path = ctx.agent.get_cgroup_path("memory", container_id)
+            io_path = ctx.agent.get_cgroup_path("blkio", container_id)
             try:
-                mem_cur_bytes = read_sysfs(mem_prefix + "memory.usage_in_bytes", int)
-                io_stats = Path(io_prefix + "blkio.throttle.io_service_bytes").read_text()
+                mem_cur_bytes = read_sysfs(mem_path / "memory.usage_in_bytes", int)
+                io_stats = (io_path / "blkio.throttle.io_service_bytes").read_text()
                 # example data:
                 #   8:0 Read 13918208
                 #   8:0 Write 0
@@ -570,6 +571,7 @@ class MemoryPlugin(AbstractComputePlugin):
     async def create_alloc_map(self) -> AbstractAllocMap:
         devices = await self.list_devices()
         return DiscretePropertyAllocMap(
+            allocation_strategy=AllocationStrategy.FILL,
             device_slots={
                 dev.device_id: DeviceSlotInfo(
                     SlotTypes.BYTES, SlotName("mem"), Decimal(dev.memory_size)
